@@ -10,7 +10,7 @@ import os,glob,copy
 import json
 
 class Background:
-    def __init__(self,container,objname='None',gfile=None,tfile='None',
+    def __init__(self,container,norder,objname='None',gfile=None,tfile='None',
                  automask=True,
                  params_bbox = {'padxleft':15,'padxright':15,'padyup':15,'halfdyup':20,'halfdylow':20,'padylow':15,'adjustx':0,'adjusty':0},
                  rescale=(False,None,None,None),
@@ -33,6 +33,7 @@ class Background:
         self.bbox = params_bbox
         self.bbox['bbox'] = self._bbox()
         self.bkg = self._bkg()
+        self.bkg.model['NORDER'] = norder
     def _bkg(self):
         rescale = (self.data['rescale'],self.data['rescale_x1'],self.data['rescale_x2'],self.data['rescale_y'])
         sigclip = (self.data['sigclip'],self.data['sigclip_niter'],self.data['sigclip_sigma'])
