@@ -1,41 +1,33 @@
 # Kornpob Bhirombhakdi
 # kbhirombhakdi@stsci.edu
+
 import os,glob,copy
 from astropy.io import fits
 from scipy.interpolate import interp1d
+from scipy.signal import savgol_filter
 import numpy as np
 import pandas as pd
 import matplotlib.pyplot as plt
+import json
 from .background import Background
 from .grismapcorr import GrismApCorr
 
 class Extract:
-    def __init__(self,grismfile,bkgfile,flatfile,tracefile,sensefile,halfdy,
+    def __init__(self,container,grismfile,sensefile,halfdy,
+                 bkgfile=None,flatfile=None,tracefile=None,
                  objname='None',
-                 savefolder='EXTRACT',overwrite=False,
-                 padyup=5,padylow=5,
-                 padxleft=15,padxright=15
+                 params = {'padyup':5,'padylow':5,'padxleft':15,'padxright':15}
                 ):
-        # remove/create savefolder
-        if overwrite:
-            os.system('rm -r {0}'.format(savefolder))
-            print('Remove {0}'.format(savefolder))
-        else:
-            tmp = os.getcwd()
-            try:
-                os.chdir(savefolder)
-                os.chdir(tmp)
-                print('Stop: Savefolder {0} already exists, overwrite = {1}'.format(savefolder,overwrite))
-                return
-            except:
-                pass
-        tmp = os.getcwd()
-        for i in savefolder.split('/'):
-            os.system('mkdir {0}'.format(i))
-            os.chdir(i)
-        os.chdir(tmp)            
-        print('Set savefolder {0}'.format(savefolder))
-        # start instantiation
+        if bkgfile is None:
+            string = './{0}/{1}_bkg.fits'.format(container.data['savefolder'],container.data['saveprefix'])
+            bkgfile = (string,1)
+        if flatfile is None:
+            string = './{0}/{1}_flat.fits'.format(container.data['savefolder'],container.data['saveprefix'])
+            flatfile = (string,0)
+        if tracefile is None:
+            string = './{0}/{1}_trace.csv'.format(container.data['savefolder'],container.data['saveprefix'])
+            tracefile = copy.deepcopy(string)
+        self.container = container
         self.data = {'objname':objname,
                      'grismfile':grismfile,
                      'bkgfile':bkgfile,
@@ -43,43 +35,146 @@ class Extract:
                      'tracefile':tracefile,
                      'sensefile':sensefile,
                      'halfdy':halfdy,
-                     'savefolder':savefolder,
-                     'padyup':padyup,
-                     'padylow':padylow,
-                     'padxleft':padxleft,
-                     'padxright':padxright
+                     'padyup':params['padyup'],
+                     'padylow':params['padylow'],
+                     'padxleft':params['padxleft'],
+                     'padxright':params['padxright']
                     }
-        self.data['ROOTNAME'] = self._rootname() 
-        self.grism = Background(objname=objname,gfile=grismfile[0],tfile=tracefile,
-                                extnum=grismfile[1],savefolder='tmp_',overwrite=True,
-                                padyup=padyup,padylow=padylow,
-                                halfdyup=halfdy,halfdylow=halfdy,
-                                padxleft=padxleft,padxright=padxright
+        self.grism = Background(container=container,objname=objname,gfile=grismfile,tfile=tracefile,
+                                params_bbox = {'padxleft':params['padxleft'],'padxright':params['padxright'],'padyup':params['padyup'],'halfdyup':halfdy,'halfdylow':halfdy,'padylow':params['padylow'],'adjustx':0,'adjusty':0},
                                )
-        self.bkg = Background(objname=objname,gfile=bkgfile[0],tfile=tracefile,
-                                extnum=bkgfile[1],savefolder='tmp_',overwrite=True,
-                                padyup=padyup,padylow=padylow,
-                                halfdyup=halfdy,halfdylow=halfdy,
-                                padxleft=padxleft,padxright=padxright
-                               )
-        self.flatfield = Background(objname=objname,gfile=flatfile[0],tfile=tracefile,
-                                extnum=flatfile[1],savefolder='tmp_',overwrite=True,
-                                padyup=padyup,padylow=padylow,
-                                halfdyup=halfdy,halfdylow=halfdy,
-                                padxleft=padxleft,padxright=padxright
+        self.bkg = Background(container=container,objname=objname,gfile=bkgfile,tfile=tracefile,
+                              params_bbox = {'padxleft':params['padxleft'],'padxright':params['padxright'],'padyup':params['padyup'],'halfdyup':halfdy,'halfdylow':halfdy,'padylow':params['padylow'],'adjustx':0,'adjusty':0},
                                )
+        self.flatfield = Background(container=container,objname=objname,gfile=flatfile,tfile=tracefile,
+                                    params_bbox = {'padxleft':params['padxleft'],'padxright':params['padxright'],'padyup':params['padyup'],'halfdyup':halfdy,'halfdylow':halfdy,'padylow':params['padylow'],'adjustx':0,'adjusty':0},
+                                   )
         self.sensitivity = self._get_sensitivity(sensefile)
     def _get_sensitivity(self,sensefile):
         tmp = fits.open(sensefile)
         return pd.DataFrame(tmp[1].data)
-    def _rootname(self):
-        grismfile = self.data['grismfile']
-        tmp = fits.open(grismfile[0])[0].header['ROOTNAME']
-        return tmp
     ##########
     ##########
     ##########
-    def compute(self,do_bkgsub=True,do_flat=True,do_apcorr=True,instrument=None,do_flam=True):
+    def save(self):
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        saveformat = self.container.data['plotformat']
+        # save self.data
+        tmp = json.dumps(self.data)
+        string = './{0}/{1}_extractdata.json'.format(savefolder,saveprefix)
+        f = open(string,'w')
+        f.write(tmp)
+        f.close()
+        print('Save {0}'.format(string))
+        # save 2d images as fits: image, bkg, flat, mask
+        primary_hdu = fits.PrimaryHDU()
+        image_hdu = fits.ImageHDU()
+        bkg_hdu = fits.ImageHDU()
+        flat_hdu = fits.ImageHDU()
+        mask_hdu= fits.ImageHDU()
+        imagebkg_hdu = fits.ImageHDU()
+        imagebkgflat_hdu = fits.ImageHDU()
+        hdul = fits.HDUList([primary_hdu,image_hdu,bkg_hdu,flat_hdu,mask_hdu,imagebkg_hdu,imagebkgflat_hdu])
+        hdul[1].header['EXTNAME'] = 'image'
+        hdul[1].data = self.grism.bkg.data['Y']
+        hdul[2].header['EXTNAME'] = 'bkg'
+        hdul[2].data = self.bkg.bkg.data['Y']
+        hdul[3].header['EXTNAME'] = 'flat'
+        hdul[3].data = self.flatfield.bkg.data['Y']
+        hdul[4].header['EXTNAME'] = 'mask'
+        hdul[4].header['Description'] = 'value = 1 for the object'
+        hdul[4].data = self.grism.bkg.data['MASK'].astype(int)
+        hdul[5].header['EXTNAME'] = 'imbkg'
+        hdul[5].header['Description'] = 'image - bkg'
+        hdul[5].data = self.grism.bkg.data['Y'] - self.bkg.bkg.data['Y']
+        hdul[6].header['EXTNAME'] = 'imbkgflat'
+        hdul[6].header['Description'] = '(image - bkg)/flat'
+        hdul[6].data = (self.grism.bkg.data['Y'] - self.bkg.bkg.data['Y'])/self.flatfield.bkg.data['Y']
+        string = './{0}/{1}_extract.fits'.format(savefolder,saveprefix)
+        hdul.writeto(string,overwrite=True)
+        print('Save {0}'.format(string))
+        # save trace as csv
+        tmp = copy.deepcopy(self.extract['trace'])
+        for i in {'XREF','YREF'}:
+            tmpp = np.full_like(tmp['XG'],None,dtype=float)
+            tmpp[0] = tmp[i]
+            tmp[i] = tmpp.copy()
+        for i in {'DYDX','DLDP'}:
+            tmpp = np.full_like(tmp['XG'],None,dtype=float)
+            for j,jj in enumerate(tmp[i]):
+                tmpp[j] = tmp[i][j]
+            tmp[i] = tmpp.copy()
+        string = './{0}/{1}_extracttrace.csv'.format(savefolder,saveprefix)
+        tmp = pd.DataFrame(tmp)
+        tmp.to_csv(string)
+        print('Save {0}'.format(string))
+        # save bbox as csv
+        tmp = copy.deepcopy(self.extract['bbox']['bbox'])
+        for i in {'BBX','BBY'}:
+            tmpp = np.full_like(tmp['XG'],None,dtype=float)
+            for j,jj in enumerate(tmp[i]):
+                tmpp[j] = tmp[i][j]
+            tmp[i] = tmpp.copy()
+        string = './{0}/{1}_extractbbox.csv'.format(savefolder,saveprefix)
+        tmp = pd.DataFrame(tmp)
+        tmp.to_csv(string)
+        print('Save {0}'.format(string))
+        # save 1d as csv
+        tmp = copy.deepcopy(self.extract['1d'])
+        for i in {'do_bkgsub','do_flat','do_apcorr'}:
+            tmpp = np.full_like(tmp['cps'],None,dtype=float)
+            tmpp[0] = self.extract[i]
+            tmp[i] = tmpp.copy()
+        for i in {'instrument'}:
+            tmpp = np.full_like(tmp['cps'],None,dtype=object)
+            tmpp[0] = self.extract[i]
+            tmp[i] = tmpp.copy()
+        string = './{0}/{1}_extract1d.csv'.format(savefolder,saveprefix)
+        tmp = pd.DataFrame(tmp)
+        tmp.to_csv(string)
+        print('Save {0}'.format(string))        
+    ##########
+    ##########
+    ##########
+    def examine(self,save=True,
+                params_extract={'central_rest':None,'width':None,'zshift':0.},
+                params_savgol={'do_savgol':True,'window_length':5,'polyorder':0},
+                plot_title=None,fontsize=12
+               ):
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        saveformat = self.container.data['plotformat']
+        central_ww = params_extract['central_rest']
+        width_ww = params_extract['width']
+        zshift = params_extract['zshift']
+        tmpww = self.extract['1d']['ww'].copy() / (1.+zshift)
+        tmpflam = self.extract['1d']['flam'].copy()
+        ww_bound = (central_ww - width_ww, central_ww + width_ww)
+        m = np.argwhere((tmpww >= ww_bound[0])&
+                        (tmpww <= ww_bound[1])).flatten()
+        tmpx = tmpww[m]
+        tmpy = tmpflam[m]
+        plt.figure()
+        plt.plot(tmpx,tmpy)
+        if params_savgol['do_savgol']:
+            window_length = params_savgol['window_length']
+            polyorder = params_savgol['polyorder']
+            plt.plot(tmpx,savgol_filter(tmpy,window_length=window_length,polyorder=polyorder))
+        plt.xlabel('rest wavelength (A), zshift = {0}'.format(zshift),fontsize=fontsize)
+        plt.ylabel('flam',fontsize=fontsize)
+        if plot_title is None:
+            plot_title = '{0}_{1}'.format(saveprefix,int(central_ww))
+        plt.title(plot_title,fontsize=fontsize)
+        plt.tight_layout()
+        if save:
+            string = './{0}/{1}_examine{3}.{2}'.format(savefolder,saveprefix,saveformat,int(central_ww))
+            plt.savefig(string,format=saveformat,bbox_inches='tight')
+            print('Save {0}'.format(string))
+    ##########
+    ##########
+    ##########
+    def compute(self,do_bkgsub=True,do_flat=True,do_apcorr=True,instrument=None,do_flam=True,fillvalue_apcorr='median'):
         data = self.grism.bkg.data['Y']
         bkg = self.bkg.bkg.data['Y']
         flat = self.flatfield.bkg.data['Y']
@@ -88,6 +183,7 @@ class Extract:
         bbox = self.grism.bbox
         ww = bbox['bbox']['WW']
         xg = bbox['bbox']['XG']
+        yg = bbox['bbox']['YG']
         # we need
         # flam = cps / (apcorr * sensitivity * ww_per_pix)
         ##### cps
@@ -108,8 +204,8 @@ class Extract:
             grismapcorr = GrismApCorr(instrument=instrument,apsize=apsize,wave=ww,
                                       aptype='diameter',apunit='pix',waveunit='A'
                                      )  
-            grismapcorr.compute()
-            apcorr = grismapcorr.data['apcorr'].diagonal()
+            grismapcorr.compute(fill_value=fillvalue_apcorr)
+            apcorr = grismapcorr.data['apcorr']
         else:
             apcorr = np.full_like(ww,1.,dtype=float)
         ##### sensitivity
@@ -131,7 +227,7 @@ class Extract:
                         'do_apcorr':do_apcorr,
                         'instrument':instrument,
                         'do_flam':do_flam,
-                        '1d':{'cps':cps,'ww':ww,'xg':xg,
+                        '1d':{'cps':cps,'ww':ww,'xg':xg,'yg':yg,
                               'apcorr':apcorr,'sensitivity':sensitivity,'ww_per_pix':ww_per_pix,
                               'flam':flam
                              }
@@ -145,7 +241,7 @@ class Extract:
     ##########
     ##########
     ##########
-    def show(self,save=False,savename_prefix=None,saveformat='pdf',
+    def show(self,save=False,
              params={'figsize':(10,10),
                      'color':'red',
                      'ls':':',
@@ -163,6 +259,9 @@ class Extract:
                      'adjust_minmax_flam':(0.9,1.1)
              }
             ):
+        saveprefix = self.container.data['saveprefix']
+        savefolder = self.container.data['savefolder']
+        saveformat = self.container.data['plotformat']
         adjust_minmax_flam = params['adjust_minmax_flam']
         minmax_im = params['minmax_im']
         xpertick_im = params['xpertick_im']
@@ -193,7 +292,6 @@ class Extract:
         cps = self.extract['1d']['cps']
         flam = self.extract['1d']['flam']
         sensitivity = self.extract['1d']['sensitivity']
-        savefolder = self.data['savefolder']
 
         fig = plt.figure(figsize=figsize)
         ax = fig.add_subplot(4,1,1)
@@ -219,7 +317,7 @@ class Extract:
         tmpmask = mask.astype(float)
         tmpmask[tmpmask==1.] = np.nan
         ax.imshow(tmpmask,origin='lower',cmap=cmap_mask,vmin=0.,alpha=alpha)
-        string = '{0} {1} \ndo_bkgsub={2} do_flat={3}'.format(self.data['objname'],self.data['ROOTNAME'],
+        string = '{0} {1} \ndo_bkgsub={2} do_flat={3}'.format(self.data['objname'],saveprefix,
                                                               self.extract['do_bkgsub'],self.extract['do_flat']
                                                              )
         ax.set_title(string,fontsize=fontsize)
@@ -288,6 +386,6 @@ class Extract:
 
         fig.tight_layout()
         if save:
-            string = './{2}/{0}_spc1d.{1}'.format(self.data['objname'],saveformat,savefolder) if savename_prefix is None else './{2}/{0}_spc1d.{1}'.format(savename_prefix,saveformat,savefolder)
+            string = './{0}/{1}_spc1d.{2}'.format(savefolder,saveprefix,saveformat)
             fig.savefig(string,format=saveformat,bbox_inches='tight')
             print('Save {0}'.format(string))
